/*
 * Project Name:
     HTTP_Demo (Net_Wireless_MCW1001 demo for MCW1001 module)
 * Target Platform:
     PIC
 * Copyright:
     (c) MikroElektronika, 2005-2012
 *
 * description  :
 *      - This code shows how to use the Net_Wireless_MCW1001 library :
 *              the board will reply to ARP & ICMP echo requests
 *              the board will reply to HTTP requests on port 80, GET method with pathnames :
 *                      /               will return the HTML main page
 *                      /s              will return board status as text string
 *                      /t0 ... /t3     will toggle RB0 to RB3 bit and return HTML main page
 *
 *      - SSID of AP is:                         mikroe_net
 *      - Password for WPA_PSK2 encryption is:   mikroe_key
 *      - When use WPA_PSK2 encryption the MRF24WB0M will calculate the PSK key (which can take up to 30 seconds).
 *
 * target devices :
 *      any PIC with integrated UART.
 *
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412D.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/eng/products/view/757/easypic-v7-development-system/
     Oscillator:      HS-PLL, 32.00000 MHz
     ext. modules:    ac:WIFI_Plus_Click on mikroBUS_2 on development board
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
     - Put jumper J5 in 3.3V position!
 
     - Pull-down PORTD.B0, PORTD.B1, PORTD.B2, PORTD.B3 at SW6, and put J17 in VCC position (board specific)
     - Close pots RA5 at J15.
     - Turn ON PORTA LEDs at SW3.
     - Place jumper J21 in RE1 position.
 */

sbit Net_Wireless_MCW1001_Chip_RST           at LATE2_bit;
sbit Net_Wireless_MCW1001_Chip_RST_Direction at TRISE2_bit;

////////////////////////////////////////////////////////////////////////////////

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;

////////////////////////////////////////////////////////////////////////////////

char txt1[]  = "mikroElektronika";
char txt2[]  = "WiFi HTTP Demo";
char txt3[]  = "example v2.1";
char txt4[]  = "Initialize";
char txt5[]  = "WiFi Module...";
char txt6[]  = "Connecting to";
char txt7[]  = "AccessPoint";
char txt8[]  = "HTTP Server is";
char txt9[]  = "running";

////////////////////////////////////////////////////////////////////////////////

// Const MAX_SIZE_OF_DATA refers to Net_Wireless_MCW1001_TCP_SendBytes function. 
// There is possible only to transmit messages to the MCW1001A in packets of max 
// 500 byte size. 
// Size of: AP header + Trailer = 11bytes  
//      (for Net_Wireless_MCW1001_TCP_SendBytes function), and because of that 
//       max size of TCP data is 500 - 11 = 489
const unsigned int MAX_TCP_PACKET_SIZE = 489;
//char txBuffer[MAX_TCP_PACKET_SIZE];

char channels[]      = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11};
char strSSID[]       = "LatonaHouse";
char wpaPassword[]   = "chickens";
char response, wifiStatus, connInProgressMark = 0;
char bindResponse, listenResponse, socketHandle, backLog, socketChild;

unsigned int remotePort, localPort;
char remoteIpAdd[4];

unsigned char   myMacAddr[] = {0x00, 0x1e, 0xc0, 0x04, 0xa6, 0xfb};    // my MAC address
unsigned char   myIpAddr[]  = {192, 168,  1, 180 };                    // my IP address
unsigned char   gwIpAddr[]  = {192, 168,  1, 1 } ;                     // gateway (router) IP address
unsigned char   ipMask[]    = {255, 255, 255,  0 } ;                   // network mask (for example : 255.255.255.0)

unsigned char strRequest[40];       // HTTP request buffer
unsigned char strTemp[9];  // Temporary buffer enough to hold 8 bit binary form

/************************************************************
 * ROM constant strings
 */
#define METHOD_LENGTH  4

unsigned char strGET[]      = "GET ";
unsigned char strStyleDoc[] = "style.css";
unsigned char strInnerDoc[] = "inner.html";
// Cascading Style Sheet: ./style.css
static const char strCSS[] =
"HTTP/1.1 200 OK\nContent-type: text/css\n\n\
body, iframe, th, td {\
font-family: \"Comic Sans MS\", \"Curlz MT\", \"Courier New\";\
font-size: 10pt;\
text-align: center;\
}";
// The main document: ./index.html
static const char strMainPage[] =
"HTTP/1.1 200 OK\nContent-type: text/html\n\n\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\
<html>\
<head>\
<meta http-equiv=\"content-type\" content=\"text/html; charset=utf-8\" />\
<meta name=\"author\" content=\"Simon Platten\" />\
<link rel=\"stylesheet\" type=\"text/css\" href=\"./style.css\" />\
<title>WiFi HTTP Demo...</title>\
<script type=\"text/javascript\"><!--\n\
var intCount = 0;\
function init() {\
window.setInterval(function() {\
if ( intCount == 0 ) {\
intCount = 30;\
var inner;\
if ( (inner = document.getElementById(\"inner\")) ) {\
var objNow = new Date();\
var strURL = \"./inner.html?t=\" + objNow.getTime();\
inner.setAttribute(\"src\", strURL);\
}\
} else {\
intCount--;\
}\
}, 100);\
};\n\
// --></script>\
</head>\
<body onload=\"init();\">\
<h1>WiFi Click HTTP Demo...</h2>\
<iframe id=\"inner\" width=\"100%\" height=\"100%\" frameborder=\"0\"></iframe>\
</body>\
</html>";
// The inner iframe document: inner.html
static const char strInnerPage[] =
"HTTP/1.1 200 OK\nContent-type: text/html\n\n\
<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\
<html>\
<head>\
<link rel=\"stylesheet\" type=\"text/css\" href=\"./style.css\" />\
</head>\
<body>\
<table border=\"1\">\
<thead>\
<tr>\
<th>ADC</th>\
<th>Port A</th>\
<th>Port B</th>\
<th>Port C</th>\
<th>Port D</th>\
<th>Port E</th>\
</tr>\
</thead>\
<tbody>\
<tr>\
<td>{$ADC}</td>\
<td>{$PORTA}</td>\
<td>{$PORTB}</td>\
<td>{$PORTC}</td>\
<td>{$PORTD}</td>\
<td>{$PORTE}</td>\
</tr>\
</tbody>\
</table>\
</body>\
</html>";
char strTxBuffer[896];

// Function which indicate state: connection to AP in progress
const char character0[] = {0,0,0,0,0,0,0,0};
const char character1[] = {0,0,0,0,0,0,16,0};
const char character2[] = {0,0,0,0,24,4,20,0};
const char character3[] = {0,0,28,2,25,5,21,0};
char rfIconMark = 1;

void DrawRfIcon() {
  char i;
  const char *ch;
  
  if(rfIconMark == 4) {
    ch = character0;
    rfIconMark = 0;
  } else {
    switch(rfIconMark) {
    case 1:
      ch = character1;
      break;

    case 2:
      ch = character2;
      break;

    case 3:
      ch = character3;
      break;
    }
  }
  rfIconMark++;
  Lcd_Cmd(64);
  for (i = 0; i<=7; i++) {
    Lcd_Chr_CP(ch[i]);
  }
  Lcd_Cmd(_LCD_RETURN_HOME);
  Lcd_Chr(2, 16, 0);
}

// Interrupt routine which increment Net_Wireless_MCW1001_Time variable every 
// second.
unsigned int wifiTmr = 0;           // timer prescaler
void interrupt() {
  if (TMR0IF_bit) {
    if( ++wifiTmr >= 122 ) {
      Net_Wireless_MCW1001_Time++;
      wifiTmr = 0;
    }
    TMR0IF_bit = 0;        // clear TMR0IF
    TMR0L = 0;             // set TMR0 for aproximetly 1 sec
  }
}

// Initialization of MCU modules
void InitMcu() {
  ANSELA = 0x20;          // ADC convertors will be used with AN4
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;
  TRISA  = 0b11110000;    // set RA5 as input for ADC, and RA0..RA3 as output
  TRISB |= 0b11111111;    // set PORTD as input for buttons
  TRISC |= 0b11111111;    // set PORTD as input for buttons
  TRISD |= 0b11111111;    // set PORTD as input for buttons
  TRISE |= 0b00001111;    // set PORTD as input for buttons
  LATA = 0;
  LATB = 0;
  LATC = 0;
  LATD = 0;
  LATE = 0;
//  TRISE |= 0b00001111;    // set PORTD as input for buttons
  SLRCON = 0;             // Set output slew rate on all ports at standard rate
  // Initialization of timer..............
  TMR0L = 0;               // Timer0 initial value
  T0CON = 0xC7;            // Set TMR0 to 8bit mode and prescaler to 256
  GIE_bit = 1;             // Enable global interrupt
  TMR0IE_bit = 1;          // Enable Timer0 interrupt

  Lcd_Init();              // Initialize Lcd
  Lcd_Cmd(_LCD_CLEAR);     // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);// Cursor off
  Lcd_Out(1,1,txt1);       // Write text in first row
  Delay_ms(2000);
  Lcd_Cmd(_LCD_CLEAR);     // Clear display
  Lcd_Out(1,2,txt2);       // Write text in first row
  Lcd_Out(2,5,txt3);       // Write text in second row
  Delay_ms(2000);
  Sound_Init(&PORTE, 1);
  Net_Wireless_MCW1001_HwReset();
}

// Initialization of WiFi module
void InitWiFi() {
 char response;

  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Out(1,1,txt4);                 // Write text in first row
  Lcd_Out(2,1,txt5);                 // Write text in second row
  Delay_ms(2000);
  
  Net_Wireless_MCW1001_TimeToWait = 1;
  response = 1;
  while(response != 0) {
    response = Net_Wireless_MCW1001_SetMode(_NET_WIRELESS_MCW1001_CP_1, 
                                     _NET_WIRELESS_MCW1001_MODE_INFRASTRUCTURE); // Set "Connection Profile 1" mode (Ad-Hoc mode)
  }
  Net_Wireless_MCW1001_TimeToWait = 5;
  Net_Wireless_MCW1001_SetChannelList(11, channels);
  Net_Wireless_MCW1001_SetSecurity_WPA(_NET_WIRELESS_MCW1001_CP_1, _NET_WIRELESS_MCW1001_WPA_PSK2, wpaPassword);    // Set WPA_PSK2 security
  Net_Wireless_MCW1001_SetSSID(_NET_WIRELESS_MCW1001_CP_1, strSSID);    // Set SSID of network

  // Set network parameters
  Net_Wireless_MCW1001_SetNetworkMask(ipMask);
  Net_Wireless_MCW1001_SetGatewayIP(gwIpAddr);
  Net_Wireless_MCW1001_SetMAC(myMacAddr);
  Net_Wireless_MCW1001_SetIP(myIpAddr);

  Net_Wireless_MCW1001_SetArpTime(1);                                   // Set gratuitous ARP timing
  Net_Wireless_MCW1001_SetRetryCount(5, 5);                             // Set number of retries for the connection
  Net_Wireless_MCW1001_SocketAllocate(2, 0, 2048, 2048, 0, 0);          // Allocate byffers for sockets
}
/**
 * Function:
 *  replace
 *
 * Parameters:
 *  pFind, what to find
 *  pReplace, the text to replace
 * pReplace should not be longer than pFind
 *
 * Returns:
 *  The number of replacements
 */
int replace(char* pFind, char* pReplace) {
  unsigned int uintLength;
  int intReplacements, i;
  char* pStart;
  char* pEnd;
  intReplacements = 0;
  uintLength = strlen(pFind);
  while( (pStart = strstr(strTxBuffer, pFind)) ) {
    pEnd = pStart + uintLength;

    for( i=0; pReplace[i]!='\0'; i++ ) {
      *pStart = pReplace[i];
      pStart++;
    }
    intReplacements++;

    if ( pStart < pEnd ) {
// Shift the remainder of the buffer
      while( *pStart != '\0' ) {
        *pStart++ = *pEnd++;
      }
    }
  }
  return intReplacements;
}

void SendHttpResponse() {
  unsigned int intReceived = 0, intSent, intTotalLength, intLength;
  char* pData, *pFile;
  
  Net_Wireless_MCW1001_TCP_ReadBytes(&socketChild, 
                                     sizeof(strRequest), 
                                     strRequest,
                                     &intReceived);
  strRequest[intReceived] = '\0';

  if ( memcmp(strRequest, strGET, METHOD_LENGTH) ) {     // only GET method is supported here
    return;
  }
// Which document has been requested?
  pFile = &strRequest[METHOD_LENGTH + 1];
  if ( memcmp(pFile, strStyleDoc, strlen(strStyleDoc)) == 0 ) {
// CSS
    for( intTotalLength=0;
         strCSS[intTotalLength]!='\0';
         intTotalLength++ ) {
      strTxBuffer[intTotalLength] = strCSS[intTotalLength];
    }
  } else if ( memcmp(pFile, strInnerDoc, strlen(strInnerDoc)) == 0 ) {
// Inner document
    for( intTotalLength=0;
         strInnerPage[intTotalLength]!='\0';
         intTotalLength++ ) {
      strTxBuffer[intTotalLength] = strInnerPage[intTotalLength];
    }
  } else {
// Default document
    for( intTotalLength=0;
         strMainPage[intTotalLength]!='\0';
         intTotalLength++ ) {
      strTxBuffer[intTotalLength] = strMainPage[intTotalLength];
    }
  }
// Build up the page and replace any macros with the actual data values
// Take the page and split it into chunks of MAX_TCP_PACKET_SIZE
    strTxBuffer[intTotalLength] = '\0';
    WordToHex(ADC_Read(4), strTemp);
    replace("{$ADC}", strTemp);
    ShortToHex(PORTA, strTemp);
    replace("{$PORTA}", strTemp);
    ShortToHex(PORTB, strTemp);
    replace("{$PORTB}", strTemp);
    ShortToHex(PORTC, strTemp);
    replace("{$PORTC}", strTemp);
    ShortToHex(PORTD, strTemp);
    replace("{$PORTD}", strTemp);
    ShortToHex(PORTE, strTemp);
    replace("{$PORTE}", strTemp);
    pData = &strTxBuffer[0];
    intTotalLength = strlen(strTxBuffer);
    
    while( intTotalLength > 0 ) {
      if ( intTotalLength >= MAX_TCP_PACKET_SIZE ) {
        intLength = MAX_TCP_PACKET_SIZE;
      } else {
        intLength = intTotalLength;
      }
      Net_Wireless_MCW1001_TCP_SendBytes(socketChild, pData, intLength, 
                                                             &intSent);
      intTotalLength -= intLength;
      pData += intLength;
    }
}

void RunHttpServer() {
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,txt8);
  Lcd_Out(2,1,txt9);
  rfIconMark = 3;
  DrawRfIcon();
  
  while(1) {
    socketHandle = 0;
    backLog = 1;
    localPort = 80;
    Net_Wireless_MCW1001_SocketCreate(&socketHandle, _NET_WIRELESS_MCW1001_SOCKET_TYPE_TCP);   // Create TCP socket
    Net_Wireless_MCW1001_SocketBind(socketHandle, &localPort, &bindResponse);                  // Bind socket to the listen port
    Net_Wireless_MCW1001_TCP_Listen(socketHandle, &backLog, &listenResponse);                  // Prepare the socket to listen for connection
                                                                                               // with one children socket.
    while(1) {
      socketChild = socketHandle;
      Net_Wireless_MCW1001_TCP_Accept(&socketChild, &remotePort, remoteIpAdd);                 // Accept incoming conncetion
      
      if (socketChild != 254) {                                                                // Accept function set socketChild.
        break;                                                                                 // If socketChild == 254 there is no incoming connection
      }
    }
    SendHttpResponse();
    Net_Wireless_MCW1001_SocketClose(socketChild);                                             // Close sockets
    Net_Wireless_MCW1001_SocketClose(socketHandle);
  }
}

void SearchSSID() {
  char markFound, numOfScanRes, response, i, strTmp[33];
  // Search SSID "mikroe_net"
  Lcd_Cmd(_LCD_CLEAR);
  Lcd_Out(1,1,"Search SSID...");
  Delay_ms(2000);
  markFound = 0;
  
  while(markFound != 1) {
    while(1) {
      response = Net_Wireless_MCW1001_ScanStart(255, &numOfScanRes);// Scan network

      if(response == 0) {                        // Check if scanning successful
        break;
      }
      Delay_ms(500);
    }
    // Check whether is there our ("mikroe_net") network
    for(i = 1; i <= numOfScanRes; i++) {
      response = Net_Wireless_MCW1001_GetScanResult(i);
      strncpy(strTmp, Net_Wireless_MCW1001_ScanResult.SSID, Net_Wireless_MCW1001_ScanResult.SSIDLength);

      if(Net_Wireless_MCW1001_ScanResult.SSIDLength > 13) {
        strTmp[14] = '\0';
        strTmp[13] = '.';
        strTmp[12] = '.';
        strTmp[11] = '.';
      } else {
        strTmp[Net_Wireless_MCW1001_ScanResult.SSIDLength] = '\0';
      }
      Lcd_Cmd(_LCD_CLEAR);
      Lcd_Out(1,1,"SSIDs Found:");
      Lcd_Out(2,1,strTmp);
      
      if(strncmp(Net_Wireless_MCW1001_ScanResult.SSID, strSSID, Net_Wireless_MCW1001_ScanResult.SSIDLength) == 0) {
        markFound = 1;
        Delay_ms(700);
        break;
      }
      Delay_ms(700);
    }
  }
}

void ConnectToAp() {
  Lcd_Cmd(_LCD_CLEAR);               // Clear display
  Lcd_Out(1,1,txt6);                 // Write text in first row
  Lcd_Out(2,1,txt7);                 // Write text in second row
  
  // Connect to AP..............................................................
  Net_Wireless_MCW1001_TimeToWait = 1;
  Net_Wireless_MCW1001_Connect(_NET_WIRELESS_MCW1001_CP_1, &wifiStatus);
  // ASCII WPA-2 pass phrase will be provided in security Key,
  // and while connecting, the MRF24WB0M will calculate the
  // PSK key (which can take up to 30 seconds).
  Net_Wireless_MCW1001_Properties.networkStatus = _NET_WIRELESS_MCW1001_STATUS_NOTCONN_STATIC_IP;
  while(1) {
    Net_Wireless_MCW1001_GetNetworkStatus();
    
    if(Net_Wireless_MCW1001_Properties.networkStatus == _NET_WIRELESS_MCW1001_STATUS_CONN_STATIC_IP) {
      break;
    }
    DrawRfIcon();
    Delay_ms(1000);
  }
  Net_Wireless_MCW1001_TimeToWait = 5;
  Sound_Play(800, 40);            // Play sound at 800Hz for 40ms
  Delay_ms(50);
  Sound_Play(800, 40);            // Play sound at 800Hz for 40ms
}

void main() {
  InitMcu();
  UART1_Init(19200);
  Delay_ms(1000);
  InitWiFi();
  SearchSSID();
  ConnectToAp();
  RunHttpServer();
}